import unittest
import markdown
from custom_attributes.plugin import read_custom, convert_hashtags, convert_text_attributes


class MyTestCase(unittest.TestCase):
    config = {
        'file': 'custom-attributes.css',
        'docs_dir': ''
    }
    maxDiff = None

    def attr_list(self, text):
        text = convert_hashtags(self.config, text)
        return markdown.markdown(text, extensions=['attr_list', 'nl2br'])

    def test_tags(self):
        """check [test/custom-attributes] with config."""
        css = read_custom(self.config)
        self.assertEqual(css, ['#left', '#blue'])

    def test_convert_line_with_attributes(self):
        """Test a simple line with **attributes**

        :arg text to left#left
        :return **text to left**{: #left}
        """
        line = 'text to left#left'
        markup_line = self.attr_list(line)
        markup_wait = '<p><strong id="left">text to left</strong></p>'

        self.assertEqual(markup_line, markup_wait)

    def test_convert_with_hashtags(self):
        """Test a simple hashtags.

        :arg #FFXIV
        :return: **FFXIV**{: #FFXIV .hash}
        """
        line = '#FFXIV'
        marked = self.attr_list(line)
        expected = '<p><strong class="hash" id="FFXIV">FFXIV</strong></p>'
        self.assertEqual(marked, expected)

    def test_double_hashtags(self):
        """Test a simple line with multiple hashtags.

        :arg #FFXIV #other
        :return: **FFXIV**{: #FFXIV .hash} **other**{: #other .hash}
        """
        line = '#FFXIV #other'
        markup_expected = '<p><strong class="hash" id="FFXIV">FFXIV</strong> <strong class="hash" id="other">other</strong></p>'
        markup_tested = self.attr_list(line)
        self.assertEqual(markup_expected,
                         markup_tested)

    def test_double_attributes(self):
        """Tests a multiple but same attributes.

        :arg text1#left text2#left
        :return: **text1 text2**{: #left}
        """
        line = 'text1#left text2#left'
        markup_expected = '<p><strong id="left">text1 text2</strong></p>'
        markup_tested = self.attr_list(line)
        self.assertEqual(markup_tested, markup_expected)

    def test_multiple_attributes(self):
        """Test a text with two differents attributes. Note that attributes
        lists can be applicated on entire paragraph or simple text.

        :arg Lorem ipsum dolor#blue sit amet, consectetur adipiscing elit#left
        :return: **Lorem ipsum **dolor**{: #blue} sit amet, consectetur adipiscing elit**{: #left}
        """
        line = 'Lorem ipsum dolor#blue sit amet, consectetur adipiscing elit#left'
        expected_markup = '<p><strong>Lorem ipsum </strong>dolor<strong id="left">{: #blue} sit amet, consectetur adipiscing elit</strong></p>'
        tested_markup = self.attr_list(line)
        self.assertEqual(tested_markup, expected_markup)

    def test_attributes_plus_tags(self):
        """Test an attributes coupled with a tag.

        :arg to left#left #FFXIV
        :return: to **left**{: #left} **FFXIV**{: #FFXIV .hash}
        """
        line = 'to left#left #FFXIV'
        expected_markup = '<p>to <strong id="left">left</strong> <strong class="hash" id="FFXIV">FFXIV</strong></p>'
        tested_markup = self.attr_list(line)
        self.assertEqual(expected_markup, tested_markup)

    def test_long_line_tagged(self):
        """Test a simple line with tags + attributes. Note the space and no
        word before the last tags.

        :arg lorem ipsum with #FFXIV and #left
        :return: lorem ipsum with **FFXIV**{: #FFXIV .hash} and \n{: #left}
        """
        line = 'lorem ipsum with #FFXIV and #left'
        expected_markup = '<p id="left">lorem ipsum with <strong class="hash" id="FFXIV">FFXIV</strong> and <br /></p>'
        tested_markup = self.attr_list(line)
        self.assertEqual(expected_markup, tested_markup)

    def test_entire_text(self):
        """Test a lorem ipsum with tags and attributes."""
        text = '#Lorem ipsum dolor sit amet, consectetur adipiscing elit.\n\nLorem ipsum dolor sit amet, #consectetur adipiscing elit. Morbi sollicitudin elementum vulputate. Etiam risus massa, fringilla in vestibulum id, consequat vitae metus. Sed nulla dui, finibus dapibus suscipit #nec, cursus rutrum ante. In hac habitasse platea dictumst. Duis accumsan cursus arcu eget congue. #Vestibulum ante ipsum primis in faucibus orci.#left'
        expected_markup = '<p><strong class="hash" id="Lorem">Lorem</strong> ipsum dolor sit amet, consectetur adipiscing elit.</p>\n<p><strong>Lorem ipsum dolor sit amet, </strong>consectetur<strong>{: #consectetur .hash} adipiscing elit. Morbi sollicitudin elementum vulputate. Etiam risus massa, fringilla in vestibulum id, consequat vitae metus. Sed nulla dui, finibus dapibus suscipit </strong>nec<strong>{: #nec .hash}, cursus rutrum ante. In hac habitasse platea dictumst. Duis accumsan cursus arcu eget congue. </strong>Vestibulum<strong id="left">{: #Vestibulum .hash} ante ipsum primis in faucibus orci.</strong></p>'
        tested_markup = markdown.markdown(convert_text_attributes(
            text, self.config).strip(), extensions=['attr_list', 'nl2br'])
        self.assertEqual(expected_markup.strip(), tested_markup.strip())



"""Main script that convert the #hashtags of a markdown text during the
on_page_markdown event."""

import re
from pathlib import Path

from mkdocs.config import Config, config_options
from mkdocs.plugins import BasePlugin
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page


def read_custom(config: dict[str, str]) -> list:
    """Read the css file and take each css ID selector and return it as a
    list."""
    css_file = Path(config.get('docs_dir'), config.get('file'))
    css = []
    try:
        with open(css_file, 'r', encoding='utf-8') as custom_attr:
            for i in custom_attr.readlines():
                if i.startswith('#'):
                    css.append(i.replace('{\n', '').strip())
    except FileNotFoundError:
        print('No CSS configured.')
        return []
    return css


def cleanned_word(line: str, word_regex: str) -> str:
    """Check the word before the attributes tags"""
    word_before_tags = re.search(word_regex, line).group().strip() if re.search(word_regex,
                                                                                line) else ''
    return word_before_tags


def convert_hashtags(config: dict[str, str], line: str) -> str:
    """Convert the tags attributes to the list attributes when reading a
    line."""
    css = read_custom(config)
    token = re.findall(r'#\w+', line)
    token = list(set(token))
    for tag in token:
        if tag in css:
            clean_line = line.replace(tag, '')

            if len(clean_line.strip()) == 0:
                return line
            markup = '**{: ' + tag + '}'
            word_regex = r'\w+'+re.escape(tag)

            if line.startswith('#'):
                heading = re.search('^#*', line).group() + ' '
                without_heading = re.sub('^#*', '', line).strip()
                word_before_tags = cleanned_word(
                    without_heading, word_regex)

                replaced_tags = '**' + word_before_tags.replace(tag, markup)
                ial = heading + re.sub(word_regex, replaced_tags, without_heading)
            else:
                word_before_tags = cleanned_word(line, word_regex)
                replaced_tags = '**' + word_before_tags.replace(tag, markup)
                ial = re.sub(word_regex, replaced_tags, line)

            if line.strip().rstrip().lstrip().replace('\n', '').endswith(tag):
                markup = markup.replace('**', '')
                word_regex = r'\S+'+re.escape(tag)
                if line.startswith('#'):
                    ial = clean_line + ' ' + markup
                else:
                    word_before_tags = cleanned_word(line, word_regex)
                    if word_before_tags == '' or any(selector in line for selector in token):
                        ial = clean_line + '\n' + markup
                        ial = ial
                    else:
                        ial = '**' + clean_line + '**' + markup
                        ial = ial

            line = ial.strip()
        else:
            ial = (
                '**'
                + tag.replace('#', ' ').strip()
                + '**{: '
                + tag.strip()
                + ' .hash}'
            )
            line = line.replace(tag, ial, 1)
    return line


def convert_text_attributes(markdown: str, config: dict[str, str]) -> str:
    """Read an entire text to convert the tags attributes to the list
    attributes."""
    files_contents = markdown.split('\n')
    markdown = ''
    code_blocks = False
    for line in files_contents:
        if code_blocks and line.startswith('```'):
            code_blocks = False
        elif line.startswith('```'):
            code_blocks = True
        elif re.search(r'#\w+', line) and not re.search(
                r'(`|\[{2}|\()(.*)#(.*)(`|]{2}|\))', line
        ) and not code_blocks:
            line = convert_hashtags(config, line)
        markdown += line + '\n'
    return markdown


class TagsAttributePlugins(BasePlugin):
    """Reads the files and convert #tags to **tags**{: #tags .hash} Convert to
    id/attributes list if found in configuration files."""
    config_scheme = (
        ('file', config_options.Type(str, default='assets/css/custom_attributes.css')),
    )

    def on_page_markdown(self, markdown: str, page: Page, config: Config, files: Files) -> str:
        """Run the conversion based on the page_markdown event."""
        config={
            'docs_dir': config['docs_dir'],
            'file' : self.config['file']
        }
        return convert_text_attributes(markdown, config)



from setuptools import setup, find_packages

version = '0.1.6'

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

with open("requirements.txt") as f:
    required = f.read().splitlines()

setup(
    name='mkdocs-custom-tags-attributes',
    version=version,
    description='Adding custom attributes using hashtags.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Mara-Li/mkdocs-custom-tags-attributes',
    author='Mara-Li',
    author_email='mara-li@outlook.fr',
    license='GPL-3',
    python_requires='>=3.10',
    install_requires=required,
    tests_require=['pytest'],
    packages=find_packages(),
    keywords="mkdocs, custom attributes, markdown extension, markdown, md, attribute list, attr_list",
    entry_points={'mkdocs.plugins': ['custom-attributes = custom_attributes.plugin:TagsAttributePlugins']}
)
